@@ -1,4 +1,4 @@
-/* $Id: cut.c,v 1.27.2.3 2001/05/10 22:55:16 astyanax Exp $ */
+/* $Id: cut.c,v 1.27.2.9 2001/06/22 21:47:23 astyanax Exp $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
@@ -61,7 +61,7 @@ void add_to_cutbuffer(filestruct * inptr)
 void cut_marked_segment(filestruct * top, int top_x, filestruct * bot,
 			int bot_x)
 {
-    filestruct *tmp, *next;
+    filestruct *tmp, *next, *botcopy;
     char *tmpstr;
 
     /* Set up the beginning of the cutbuffer */
@@ -103,7 +103,15 @@ void cut_marked_segment(filestruct * top, int top_x, filestruct * bot,
      * up a newline when we're grabbing the last line of the mark.  For
      * the same reason, we don't do an extra totsize decrement. */
 
-    add_to_cutbuffer(bot);
+
+    /* I honestly do not know why this is needed.  After many hours of
+       using gdb on an OpenBSD box, I can honestly say something is 
+       screwed somewhere.  Not doing this causes update_line to annihilate
+       the last line copied into the cutbuffer when the mark is set ?!?!? */
+    botcopy = copy_node(bot);
+    add_to_cutbuffer(botcopy);
+    free(bot);
+
     top->next = next;
     if (next != NULL)
 	next->prev = top;
@@ -134,6 +142,7 @@ int do_cut_text(void)
     int newsize, cuttingtoend = 0;
 #endif
 
+
     check_statblank();
     if (fileptr == NULL || fileptr->data == NULL)
 	return 0;
@@ -144,6 +153,7 @@ int do_cut_text(void)
 	free_filestruct(cutbuffer);
 	cutbuffer = NULL;
 
+	marked_cut = 0;
 #ifdef DEBUG
 	fprintf(stderr, _("Blew away cutbuffer =)\n"));
 #endif
@@ -156,6 +166,23 @@ int do_cut_text(void)
 #ifndef NANO_SMALL
     if (ISSET(CUT_TO_END) && !ISSET(MARK_ISSET)) {
 	if (current_x == strlen(current->data)) {
+
+    	/* If the line is empty and we didn't just cut a non-blank
+		line, create a dummy line and add it to the cutbuffer */
+	    if (marked_cut != 1 && current->next != filebot) {
+
+		filestruct *junk;
+
+		junk = NULL;
+		junk = make_new_node(current);
+	        junk->data = nmalloc(1 * sizeof (char));
+		junk->data[0] = 0;
+
+		add_to_cutbuffer(junk);
+		dump_buffer(cutbuffer);
+
+	    }
+
 	    do_delete();
 	    SET(KEEP_CUTBUFFER);
 	    marked_cut = 2;
@@ -301,6 +328,8 @@ int do_uncut_text(void)
 	    current->data = tmpstr;
 	    current_x += strlen(cutbuffer->data);
 	    totsize += strlen(cutbuffer->data);
+	    if (strlen(cutbuffer->data) == 0)
+		totlines++;
 
 	    placewewant = xplustabs();
 	    update_cursor();
@@ -361,7 +390,7 @@ int do_uncut_text(void)
 	   screw up all the work we just did and separate the line.  There
 	   must be a better way to do this, but not at 1AM on a work night. */
 
-	if (marked_cut == 2 && current_x != strlen(current->data)) {
+	if (marked_cut == 2) {
 	    tmp = make_new_node(current);
 	    tmp->data = nmalloc(strlen(&current->data[current_x]) + 1);
 	    strcpy(tmp->data, &current->data[current_x]);
@@ -370,6 +399,10 @@ int do_uncut_text(void)
 	    current = current->next;
 	    current_x = 0;
 	    placewewant = 0;
+
+	    /* Extra line added, update stuff */
+	    totlines++;
+	    totsize++;
 	}
 	/* Renumber from BEFORE where we pasted ;) */
 	renumber(hold);
@@ -378,6 +411,7 @@ int do_uncut_text(void)
 	dump_buffer(cutbuffer);
 	set_modified();
 	edit_refresh();
+	UNSET(KEEP_CUTBUFFER);
 	return 0;
 #else
     if (0) {
