@@ -1,4 +1,4 @@
-/* $Id: nano.c,v 1.144.2.3 2001/05/05 17:38:31 astyanax Exp $ */
+/* $Id: nano.c,v 1.144.2.7 2001/07/01 04:28:45 astyanax Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -1391,6 +1391,7 @@ int do_alt_speller(char *file_name)
     int alt_spell_status;
     pid_t pid_spell;
     char *ptr;
+    long lineno_cur = current->lineno;
     static int arglen = 3;
     static char **spellargs = (char **) NULL;
 
@@ -1440,8 +1441,8 @@ int do_alt_speller(char *file_name)
     free_filestruct(fileage);
     global_init();
     open_file(file_name, 0, 1);
-    edit_update(fileage, CENTER);
-    display_main_list();
+
+    do_gotoline(lineno_cur);
     set_modified();
 
     return TRUE;
@@ -1601,25 +1602,24 @@ RETSIGTYPE handle_hup(int signal)
 /* What do we do when we catch the suspend signal */
 RETSIGTYPE do_suspend(int signal)
 {
-
-    act.sa_handler = SIG_DFL;
-    sigemptyset(&act.sa_mask);
-    sigaction(SIGTSTP, &act, NULL);
-
     endwin();
-    fprintf(stderr, "\n\n\n\n\nUse \"fg\" to return to nano\n");
-    raise(SIGTSTP);
+    printf("\n\n\n\n\nUse \"fg\" to return to nano\n");
+    fflush(stdout);
+
+    /* We used to re-enable the default SIG_DFL and raise SIGTSTP, but 
+	then we could be (and were) interrupted in the middle of the call.
+	So we do it the mutt way instead */
+    kill(0, SIGSTOP);
 }
 
 /* Restore the suspend handler when we come back into the prog */
 RETSIGTYPE do_cont(int signal)
 {
 
-    act.sa_handler = do_suspend;
-    sigemptyset(&act.sa_mask);
-    sigaction(SIGTSTP, &act, NULL);
-    initscr();
-    total_refresh();
+    /* Now we just update the screen instead of having to reenable the
+	SIGTSTP handler */
+
+    doupdate();
 }
 
 void handle_sigwinch(int s)
@@ -1698,23 +1698,15 @@ void handle_sigwinch(int s)
 
 void signal_init(void)
 {
+#ifdef _POSIX_VDISABLE
+    struct termios term;
+#endif
 
     /* Trap SIGINT and SIGQUIT cuz we want them to do useful things. */
     memset(&act, 0, sizeof(struct sigaction));
     act.sa_handler = SIG_IGN;
     sigaction(SIGINT, &act, NULL);
 
-    if (!ISSET(SUSPEND)) {
-	sigaction(SIGTSTP, &act, NULL);
-    } else {
-	act.sa_handler = do_suspend;
-	sigaction(SIGTSTP, &act, NULL);
-
-	act.sa_handler = do_cont;
-	sigaction(SIGCONT, &act, NULL);
-    }
-
-
     /* Trap SIGHUP cuz we want to write the file out. */
     act.sa_handler = handle_hup;
     sigaction(SIGHUP, &act, NULL);
@@ -1722,6 +1714,30 @@ void signal_init(void)
     act.sa_handler = handle_sigwinch;
     sigaction(SIGWINCH, &act, NULL);
 
+    if (!ISSET(SUSPEND)) {
+
+/* Insane! */
+#ifdef _POSIX_VDISABLE
+	tcgetattr(0, &term);
+	term.c_cc[VSUSP] = _POSIX_VDISABLE;
+	tcsetattr(0, TCSANOW, &term);
+#else
+	act.sa_handler = SIG_IGN;
+	sigaction(SIGTSTP, &act, NULL);
+#endif
+
+    } else {
+	/* if we don't do this, it seems other stuff interrupts the
+	   suspend handler!  Try using nano with mutt without this line */
+	sigfillset(&act.sa_mask);
+
+	act.sa_handler = do_suspend;
+	sigaction(SIGTSTP, &act, NULL);
+
+	act.sa_handler = do_cont;
+	sigaction(SIGCONT, &act, NULL);
+    }
+
 }
 
 void window_init(void)
